/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.Image;
import java.io.*;

public class Bitmap extends Resource
{
  public static final int IMAGE_CHANGED=100;
  public static final int PARENT_DIR_CHANGED=101;

  public static final int ICON=10;
  public static final int SMALLICON=11;
  public static final int BITMAP=12;

  public static final int BW=0;
  public static final int GREY=1;
  public static final int GREY16=2;
  public static final int COLOR=3;
  public static final int COLOR16=4;
  public static final int FAMILY=5;
  public static final int FAMILYSPECIAL=6;
  public static final int NUM_TYPES=7;

  public static final int ICON_BW=ICON*NUM_TYPES+BW;
  public static final int ICON_GREY=ICON*NUM_TYPES+GREY;
  public static final int ICON_GREY16=ICON*NUM_TYPES+GREY16;
  public static final int ICON_COLOR=ICON*NUM_TYPES+COLOR;
  public static final int ICON_FAMILY=ICON*NUM_TYPES+FAMILY;

  public static final int SMALLICON_BW=SMALLICON*NUM_TYPES+BW;
  public static final int SMALLICON_GREY=SMALLICON*NUM_TYPES+GREY;
  public static final int SMALLICON_GREY16=SMALLICON*NUM_TYPES+GREY16;
  public static final int SMALLICON_COLOR=SMALLICON*NUM_TYPES+COLOR;
  public static final int SMALLICON_FAMILY=SMALLICON*NUM_TYPES+FAMILY;

  public static final int BITMAP_BW=BITMAP*NUM_TYPES+BW;
  public static final int BITMAP_GREY=BITMAP*NUM_TYPES+GREY;
  public static final int BITMAP_GREY16=BITMAP*NUM_TYPES+GREY16;
  public static final int BITMAP_COLOR=BITMAP*NUM_TYPES+COLOR;
  public static final int BITMAP_COLOR16=BITMAP*NUM_TYPES+COLOR16;
  public static final int BITMAP_FAMILY=BITMAP*NUM_TYPES+FAMILY;
  public static final int BITMAP_FAMILYSPECIAL=BITMAP*NUM_TYPES+FAMILYSPECIAL;

  public static final int NOCOMPRESS=0;
  public static final int COMPRESS=1;
  public static final int FORCECOMPRESS=2;

  public static final int NOCOLORTABLE=3;
  public static final int COLORTABLE=4;
  public static final int TRANSPARENT=5;
  public static final int TRANSPARENTINDEX=6;

  public static final String PREF_COLOR_DEPTH="Color Depth";
  public static final String PREF_BGCOLOR="Background Color";

  Project project;
  int type;
  int compress=NOCOMPRESS;
  String[] files=new String[4];
  boolean color16;
  boolean colorTable;
  int transparentIndex=-1;
  int transR=-1,transG,transB;

  public Bitmap(int type)
  {
    this.project=Project.getProject();
    this.type=type;
  }

  public Bitmap(Project project,StreamTokenizer st) throws IOException
  {
    this.project=project;
    parse(st);
  }

  public int getType()
  {
    return type;
  }

  public boolean isValid(String s)
  {
    if (s==null||s.equals(""))
      return false;
    String sl=s.toLowerCase();
    if (sl.endsWith(".bmp")||sl.endsWith(".pbm"))
    {
      File f=project.makeRelativeFile(s);
      if (f.exists())
      {
        return true;
      }
    }
    return false;
  }

  public int getCompression()
  {
    return compress;
  }

  public void setCompression(int c)
  {
    compress=c;
  }

  public boolean useColorTable()
  {
    return colorTable;
  }

  public void setColorTable(boolean b)
  {
    colorTable=b;
  }

  public void setColor16(boolean b)
  {
    if (b!=color16)
    {
      color16=b;
      fireObjectChanged(IMAGE_CHANGED);
    }
  }

  public void setFile(int i,String s)
  {
    files[i]=s;
    fireObjectChanged(IMAGE_CHANGED);
  }

  public File getFile(int i)
  {
    return project.makeRelativeFile(files[i]);
  }

  public void parentDirChanged()
  {
    for(int i=0;i<4;i++)
      if (files[i]!=null&&!files[i].equals(""))
        files[i]=project.convertToRelativeFilename(files[i]);
    fireObjectChanged(PARENT_DIR_CHANGED);
  }

  public String getTransparentString()
  {
    if (transparentIndex==-1&&transR==-1)
      return "";
    if (transparentIndex!=-1)
      return ""+transparentIndex;
    else
      return transR+","+transG+","+transB;
  }

  public boolean setTransparentString(String s)
  {
    if (s.indexOf(',')!=-1)
    {
      String[] rgb=TextTools.split(s,',');
      if (rgb.length==3)
      {
        try
        {
          int r=Integer.parseInt(rgb[0]);
          int g=Integer.parseInt(rgb[1]);
          int b=Integer.parseInt(rgb[2]);
          if ((transR!=r||transG!=g||transB!=b)&&
            r>=0&&g>=0&&b>=0&&r<256&&g<256&&b<256)
          {
            transR=r;
            transG=g;
            transB=b;
            transparentIndex=-1;
            fireObjectChanged(IMAGE_CHANGED);
            return true;
          }
        }
        catch(NumberFormatException e)
        {
          // ignore
        }
      }
    }
    else
    {
      try
      {
        int index=Integer.parseInt(s);
        if (index!=transparentIndex&&index>=0&&index<256)
        {
          transR=-1;
          transparentIndex=index;
          fireObjectChanged(IMAGE_CHANGED);
          return true;
        }
      }
      catch(NumberFormatException e)
      {
        // ignore
      }
    }
    if (transparentIndex!=-1||transR!=-1)
    {
      transparentIndex=-1;
      transR=-1;
      fireObjectChanged(IMAGE_CHANGED);
      return true;
    }
    return false;
  }

  /**
   * Gets a specified image in this bitmap
   * @param i the type of image to get: B&W,GRAY,GRAY16,COLOR
   * @return the image, or null if there is none
   */
  public Image getImage(int i)
  {
    if (!isValid(files[i]))
      return null;
    return BitmapLoader.loadImage(this,i);

    /*
    File f=project.makeRelativeFile(files[i]);
    if (i==COLOR&&transparentIndex!=-1)
      return BitmapLoader.loadImage(f,transparentIndex,null);
    else
    if (i==COLOR&&transR!=-1)
      return BitmapLoader.loadImage(f,-1,new java.awt.Color(transR,transG,transB));
    else
      return BitmapLoader.loadImage(f,(i==COLOR||(i==GREY16&&color16)?-1:-2),null);
    */
  }

  /**
   * Gets the closest image to the current colour depth in this bitmap.
   * @return the image, or null if there are no images at or below the current depth
   */
  public Image getImage()
  {
    Image im=null;
    int colorDepth=Preferences.getIntPref(PREF_COLOR_DEPTH);
    for(int i=colorDepth;i>=0&&im==null;i--)
    {
      if (i==GREY16&&color16&&colorDepth!=COLOR)
        continue;
      im=getImage(i);
    }
    return im;
  }

  public void parse(StreamTokenizer st) throws IOException
  {
    String bt=st.sval;
    int key=Keywords.get(this,bt);

    type=key/NUM_TYPES;
    int subtype=key%NUM_TYPES;
    if (subtype==COLOR16)
    {
      color16=true;
      subtype=GREY16;
    }
    else
      color16=false;

    if (type==BITMAP)
      readID(st);
    if (subtype<FAMILY)
    {
      files[subtype]=readQuotedString(st);
    }
    else
    {
      color16=subtype==FAMILYSPECIAL;
      for(int i=0;i<4;i++)
        files[i]=readQuotedString(st);
    }

    st.nextToken();
    while(st.sval!=null)
    {
      key=Keywords.get(this,st.sval);
      switch(key)
      {
        case NOCOMPRESS:
        case COMPRESS:
        case FORCECOMPRESS:
          compress=key;
          break;

        case NOCOLORTABLE:
          colorTable=false;
          break;

        case COLORTABLE:
          colorTable=true;
          break;

        case TRANSPARENT:
          st.lowerCaseMode(false);
          st.nextToken();
          transR=Expression.evaluate(st.sval);
          st.nextToken();
          transG=Expression.evaluate(st.sval);
          st.nextToken();
          transB=Expression.evaluate(st.sval);
          //st.nextToken();
          st.lowerCaseMode(true);
          break;

        case TRANSPARENTINDEX:
          st.lowerCaseMode(false);
          st.nextToken();
          transparentIndex=Expression.evaluate(st.sval);
          st.lowerCaseMode(true);
          break;

        default:
          st.pushBack();
          return;
      }
      st.nextToken();
    }
  }

  public void write(PrintStream ps)
  {
    int image=-1;
    for(int i=0;i<4;i++)
      if (files[i]!=null&&!files[i].equals(""))
      {
        if (image!=-1)
          image=color16?FAMILYSPECIAL:FAMILY;
        else
          image=i;
      }

    if (image==-1) // no images, skip
      return;

    StringBuffer sb=new StringBuffer();

    if (type==BITMAP&&image==GREY16&&color16)
      image=COLOR16;

    int cat=type*NUM_TYPES+image;

    if (image==COLOR16)
      image=GREY16;

    sb.append(Keywords.get(this,cat));
    if (type==BITMAP)
      sb.append(id.toOutputString());
    if (image<FAMILY)
      sb.append(' ').append(TextTools.quote(files[image]));
    else
    {
      for(int i=0;i<4;i++)
      {
        if (i==0||i==3)
          sb.append(Preferences.newLine).append(' ');
        sb.append(' ').append(TextTools.quote(files[i]));
      }
    }

    if (transparentIndex!=-1)
    {
      sb.append(' ').append(Keywords.get(this,TRANSPARENTINDEX));
      sb.append(' ').append(transparentIndex);
    }
    else
    if (transR!=-1)
    {
      sb.append(' ').append(Keywords.get(this,TRANSPARENT)).append(' ');
      sb.append(transR).append(' ').append(transG).append(' ').append(transB);
    }

    if (compress!=NOCOMPRESS)
      sb.append(' ').append(Keywords.get(this,compress));

    if (colorTable)
      sb.append(' ').append(Keywords.get(this,COLORTABLE));

    ps.println(sb.toString());
    if (type!=BITMAP||image>=FAMILY)
      ps.println();
  }
}