/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.io.*;
import java.util.*;

public class ResourceCollection
{
  protected String title;
  protected boolean keepPositions=false;
  protected Vector resources=new Vector();
  protected ChangeListeners listeners;

  public ResourceCollection(String title)
  {
    this.title=title;
  }

  public ResourceCollection(String title,boolean keepPositions)
  {
    this.title=title;
    keepPositions=keepPositions;
  }

  public int size()
  {
    return resources.size();
  }

  public Enumeration elements()
  {
    return resources.elements();
  }

  public void clear()
  {
    resources.removeAllElements();
    fireChanged();
  }

  public void set(int index,Resource resource)
  {
    if (resources.size()<=index)
    {
      while (resources.size()<index)
        resources.addElement(null);
      resources.addElement(resource);
    }
    else
      resources.setElementAt(resource,index);
    fireChanged();
  }

  public void add(String s)
  {
    resources.addElement(s);
    fireChanged();
  }

  public void add(Resource resource)
  {
    resources.addElement(resource);
    fireChanged();
  }

  public void remove(Resource resource)
  {
    remove(getIndexOf(resource));
    fireChanged();
  }

  public void remove(int index)
  {
    if (keepPositions)
      resources.setElementAt(null,index);
    else
      resources.removeElementAt(index);
    fireChanged();
  }

  public Resource getResourceAt(int index)
  {
    if (index>=resources.size())
      return null;
    return (Resource)resources.elementAt(index);
  }

  public int getIndexOf(Resource resource)
  {
    int i=0;
    for(Enumeration e=resources.elements();e.hasMoreElements();i++)
    {
      Resource r=(Resource)e.nextElement();
      if (r!=null&&r.equals(resource))
        return i;
    }
    return -1;
  }

  public Resource getResource(int id)
  {
    for(Enumeration e=resources.elements();e.hasMoreElements();)
    {
      Resource r=(Resource)e.nextElement();
      //System.out.println(r);
      if (r!=null&&r.getIDValue()==id)
        return r;
    }
    return null;
  }

  public void write(PrintStream ps)
  {
    if (resources.size()==0)
      return;
    if (title!=null)
      ps.println("// "+title);
    for(Enumeration e=resources.elements();e.hasMoreElements();)
    {
      Object o=e.nextElement();
      if (o instanceof Resource)
        ((Resource)o).write(ps);
      else
      if (o!=null)
        ps.println(o);
      //ps.println();
    }
    ps.println();

  }

  public void resourceIDChanged(Resource resource)
  {
    fireChanged();
  }

  public void addChangeListener(ChangeListener listener)
  {
    if (listeners==null)
      listeners=new ChangeListeners();
    listeners.add(listener);
  }

  public void removeChangeListener(ChangeListener listener)
  {
    if (listeners==null)
      return;
    listeners.remove(listener);
  }

  public void fireChanged()
  {
    if (listeners!=null)
      listeners.fireObjectChanged(this,ChangeListener.MODIFIED);
  }

}